@@ -1985,6 +1985,20 @@ int aug_transform(struct augeas *aug, const char *lens,
     return result;
 }
 
+int aug_escape_name(augeas *aug, const char *in, char **out) {
+    int result;
+
+    api_entry(aug);
+    ARG_CHECK(in == NULL, aug, "aug_escape_name: IN must not be NULL");
+    ARG_CHECK(out == NULL, aug, "aug_escape_name: OUT must not be NULL");
+
+    result = pathx_escape_name(in, out);
+    ERR_NOMEM(result < 0, aug);
+ error:
+    api_exit(aug);
+    return result;
+}
+
 int aug_print(const struct augeas *aug, FILE *out, const char *pathin) {
     struct pathx *p;
     int result;
@@ -2031,7 +2045,7 @@ void aug_close(struct augeas *aug) {
 
 int __aug_load_module_file(struct augeas *aug, const char *filename) {
     api_entry(aug);
-    int r = load_module_file(aug, filename);
+    int r = load_module_file(aug, filename, NULL);
     api_exit(aug);
     return r;
 }
