@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 2.5.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
 /* Bison interface for Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
-   
+
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,36 +26,44 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_AUGL_PARSER_H_INCLUDED
+# define YY_AUGL_PARSER_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int augl_debug;
+#endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     DQUOTED = 258,
-     REGEXP = 259,
-     LIDENT = 260,
-     UIDENT = 261,
-     QIDENT = 262,
-     ARROW = 263,
-     KW_MODULE = 264,
-     KW_AUTOLOAD = 265,
-     KW_LET = 266,
-     KW_LET_REC = 267,
-     KW_IN = 268,
-     KW_STRING = 269,
-     KW_REGEXP = 270,
-     KW_LENS = 271,
-     KW_TEST = 272,
-     KW_GET = 273,
-     KW_PUT = 274,
-     KW_AFTER = 275
-   };
+  enum yytokentype
+  {
+    DQUOTED = 258,
+    REGEXP = 259,
+    LIDENT = 260,
+    UIDENT = 261,
+    QIDENT = 262,
+    ARROW = 263,
+    KW_MODULE = 264,
+    KW_AUTOLOAD = 265,
+    KW_LET = 266,
+    KW_LET_REC = 267,
+    KW_IN = 268,
+    KW_STRING = 269,
+    KW_REGEXP = 270,
+    KW_LENS = 271,
+    KW_TEST = 272,
+    KW_GET = 273,
+    KW_PUT = 274,
+    KW_AFTER = 275
+  };
 #endif
 /* Tokens.  */
 #define DQUOTED 258
@@ -77,15 +85,12 @@
 #define KW_PUT 274
 #define KW_AFTER 275
 
-
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
-
-/* Line 2068 of yacc.c  */
-#line 89 "parser.y"
+#line 89 "parser.y" /* yacc.c:1909  */
 
   struct term    *term;
   struct type    *type;
@@ -99,37 +104,31 @@ typedef union YYSTYPE
   int            intval;
   enum quant_tag quant;
 
-
-
-/* Line 2068 of yacc.c  */
-#line 106 "parser.h"
-} YYSTYPE;
+#line 108 "parser.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
-
-
+/* Location type.  */
 #if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
-typedef struct YYLTYPE
+typedef struct YYLTYPE YYLTYPE;
+struct YYLTYPE
 {
   int first_line;
   int first_column;
   int last_line;
   int last_column;
-} YYLTYPE;
-# define yyltype YYLTYPE /* obsolescent; will be withdrawn */
+};
 # define YYLTYPE_IS_DECLARED 1
 # define YYLTYPE_IS_TRIVIAL 1
 #endif
 
 
 
+int augl_parse (struct term **term, yyscan_t scanner);
 /* "%code provides" blocks.  */
-
-/* Line 2068 of yacc.c  */
-#line 46 "parser.y"
+#line 46 "parser.y" /* yacc.c:1909  */
 
 #include "info.h"
 
@@ -140,7 +139,6 @@ struct state {
 };
 
 
+#line 143 "parser.h" /* yacc.c:1909  */
 
-
-/* Line 2068 of yacc.c  */
-#line 147 "parser.h"
+#endif /* !YY_AUGL_PARSER_H_INCLUDED  */
